//
//  PTImageIOProtocols.h
//  PTImageIO
//
//  Created by Rimas Mickevičius on 2013-01-03.
//  Copyright (c) 2013 Rimas Mickevičius. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreGraphics/CoreGraphics.h>
#import <PTImageIO/PTBitmapDataInfo.h>

@class PTImageIODataWriteContext;
@class PTImageIODataReadContext;

//--------------------------------------------------------------------------------------------------------------------
// Data State Info Collector Protocol
//--------------------------------------------------------------------------------------------------------------------
@protocol PTImageIODataStateInfoCollectorProtocol;
typedef BOOL (^PTIMageIODataStateLayersInfoProvideBlock)(id <PTImageIODataStateInfoCollectorProtocol> layerDataCollector);

@protocol PTImageIODataStateInfoCollectorProtocol <NSObject>
- (NSDictionary *)requiredDocumentInfoKeysInformationDictionary;
- (NSArray *)requiredDocumentInfoKeys;
- (BOOL)customDocumentInfoKeysAreAllowed;
- (BOOL)setDocumentValue:(id)documentValue forKey:(NSString *)documentInfoKey;
//-(void)iterateOverRequiredDocumentKeysUsingBlock:(PTImageIODataSaveStateSnapshotRequiredValuesIteratorBlock)iteratorBlock;
- (BOOL)requiresLayersInformation;
- (NSDictionary *)requiredLayerInfoKeysInformationDictionary;
- (NSArray *)requiredLayerInfoKeys;
- (BOOL)customLayerInfoKeysAreAllowed;
- (BOOL)beginProvidingInfoForLayerWithID:(NSString *)layerIDstring ofType:(NSString *)layerTypeUTI parentID:(NSString *)layerParentIDstring indexAtParent:(NSInteger)indexAtParent continueProvidingInfoWithBlock:(PTIMageIODataStateLayersInfoProvideBlock)infoProvideBlock;
- (BOOL)setLayerValue:(id)layerValue forKey:(NSString* )layerInfoKey;
- (BOOL)requiresPrerenderedData;
@end
//--------------------------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------------------------
// Data State Info Provider Protocol
//--------------------------------------------------------------------------------------------------------------------
#define PTImageIODataStateInfoProviderPrerenderedPreviewImageDataKey    @"PTImageIODataStateInfoProviderPrerenderedPreviewImageDataKey"
@protocol PTImageIODataStateInfoProviderProtocol <NSObject>
- (NSString *)dataProviderIDForWritingContext:(PTImageIODataWriteContext *)writeContext;
- (BOOL)populateInfoIntoDataCollector:(id <PTImageIODataStateInfoCollectorProtocol>)dataCollector context:(PTImageIODataWriteContext *)writeContext;
- (void)fillBuffer:(PTBitmapDataBuffer *)layerBitmapDataBufferRef withNonCopiedDataOfLayerWithID:(NSString *)layerIDstring context:(PTImageIODataWriteContext *)writeContext;
- (BOOL)providerSupportsBitmapDataCopyOnWriteProtectionForContext:(PTImageIODataWriteContext *)writeContext;
- (NSData *)nonCopiedRGBADataForPrerenderedImageOfSize:(CGSize *)renderedImageBufferSizeRef rowBytes:(NSInteger *)renderedImageBufferRowbytesRef context:(PTImageIODataWriteContext *)writeContext;
#if PT_TARGET_IOS
- (PTBitmapDataBufferWrapper *)noncopiedPrerenderedImageBufferWrapperForContext:(PTImageIODataWriteContext *)writeContext;
#endif
- (BOOL)providerSupportsAsynchronousPreviewDataForContext:(PTImageIODataWriteContext *)writeContext;
- (void)performCleanupDueToFinishedStateWritingForContext:(PTImageIODataWriteContext *)writeContext;
@optional
#if PT_TARGET_OSX
- (PTBitmapDataBufferWrapper *)noncopiedPrerenderedImageBufferWrapperForContext:(PTImageIODataWriteContext *)writeContext;
#endif
- (void)unblockUserInteractionAndContinueForContext:(PTImageIODataWriteContext *)writeContext;
- (void)finishedUsingDataOfLayerWithID:(NSString *)layerIDstring context:(PTImageIODataWriteContext *)writeContext;
@end
//--------------------------------------------------------------------------------------------------------------------



//--------------------------------------------------------------------------------------------------------------------
// File Data Info Provider Protocol
//--------------------------------------------------------------------------------------------------------------------
@protocol PTImageIOFileDataInfoProviderProtocol;
typedef BOOL (^PTIMageIOFileDataAvailableDocumentKeysProviderBlock)(NSString *documentKey, id documentValue);
typedef BOOL (^PTImageIOFileLayersIterationBlock)(NSString *layerID, NSString *layerType, NSString *layerParentID, NSInteger layerIndexAtParent, id <PTImageIOFileDataInfoProviderProtocol> additionalLayerInfoprovider);
typedef BOOL (^PTIMageIOFileDataAdditionalLayerInfoProviderBlock)(NSString *layerInfoKey, id layerValue);

@protocol PTImageIOFileDataInfoProviderProtocol <NSObject>
- (NSArray *)allAvailableDocumentInfoKeys;
- (void)enumerateAvailableDocumentInfo:(PTIMageIOFileDataAvailableDocumentKeysProviderBlock)infoProviderBlock;
- (id)documentValueForKey:(NSString *)documentInfoKey;

- (BOOL)providesLayerInformation;
- (BOOL)enumerateLayersUsingBlock:(PTImageIOFileLayersIterationBlock)addLayerBlock;
- (NSArray *)availableAdditionalLayerInfoKeysForCurrentLayer;
- (BOOL)enumerateAdditionalLayerInfoUsingBlock:(PTIMageIOFileDataAdditionalLayerInfoProviderBlock)additionalInfoProvideBlock;
- (id)additionalLayerValueForKey:(NSString *)additionalLayerInfoKey;

@end
//--------------------------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------------------------
// File Data Open Info Consumer
//--------------------------------------------------------------------------------------------------------------------

typedef NS_ENUM(NSInteger, PTImageIOConsumerNeedsPreviewImageDataResponse) {
    PTImageIOConsumerNeedsPreviewImageDataNever             = 0,
    PTImageIOConsumerNeedsPreviewImageDataAlways            = 1,
    PTImageIOConsumerNeedsPreviewImageDataIfLayersFailOnly  = 2,
    PTImageIOConsumerNeedsPreviewImageDataDefaultResponse   = PTImageIOConsumerNeedsPreviewImageDataNever,
};

@interface PTImageIODataConsumerDocumentSupportRequest : NSObject

- (instancetype)initWithUTI:(NSString *)utiString version:(id)version size:(CGSize)size colorSpace:(CGColorSpaceRef)colorSpace promisedMemoryAmount:(unsigned long long)memoryAmount bitmapLayersCount:(NSUInteger)bitmapLayersCount vectorLayersCount:(NSUInteger)vectorLayersCount textlayersCount:(NSUInteger)textLayersCount groupLayersCount:(NSUInteger)groupLayersCount layerMasksCount:(NSUInteger)layerMasksCount;

@property   (readonly)  NSString            *documentUTI;
@property   (readonly)  id                  documentVersion;
@property   (readonly)  CGSize              documentSize;
@property   (readonly)  CGColorSpaceRef     documentColorSpace;
@property   (readonly)  unsigned long long  memoryAmount;
@property   (readonly)  NSUInteger          bitmapLayersCount;
@property   (readonly)  NSUInteger          vectorLayersCount;
@property   (readonly)  NSUInteger          textLayersCount;
@property   (readonly)  NSUInteger          groupLayersCount;
@property   (readonly)  NSUInteger          layerMasksCount;

@end



@interface PTImageIODataConsumerDocumentSupportResponse : NSObject



@end

@protocol PTImageIOFileDataOpenInfoConsumer <NSObject>
- (NSString *)dataConsumerID;

- (PTImageIODataConsumerDocumentSupportResponse *)requestDocumentSupportWith:(PTImageIODataConsumerDocumentSupportRequest *)supportRequest context:(PTImageIODataReadContext *)readContext;








- (BOOL)readDocumentInfoFrom:(id <PTImageIOFileDataInfoProviderProtocol>)fileInfoDataProvider context:(PTImageIODataReadContext *)readContext error:(NSError **)outError;
// if you are going to take ownership of bitmapDataBufferRef->dataBuffer.data, you must set it to NULL (best way to do that is to call PTBitmapDataBufferCleanup with second parameter NO) before returning. Otherwise the caller will free buffer memory after return
- (BOOL)applyBitmapDataBuffer:(PTBitmapDataBuffer *)bitmapDataBufferRef forLayerWithID:(NSString *)layerIDstring context:(PTImageIODataReadContext *)readContext;
- (PTBitmapDataBuffer *)initializedPreviewImageBufferForSize:(CGSize)previewImageSize;
#if PT_TARGET_IOS
/**
 * Returns Core Video memory buffer for direct manipulations.
 * You MUST call stopAccessingBufferOfLayerWithID:context: after finished using returned buffer.
 */
- (BOOL)configureBuffer:(PTBitmapDataBuffer *)bufferRef forAccessingBufferOfLayerWithID:(NSString *)layerIDString context:(PTImageIODataReadContext *)readContext;
/**
 * Unlocks previously locked texture in configureBuffer:forAccessingBufferOfLayerWithID:context:
 */
- (BOOL)stopAccessingBufferOfLayerWithID:(NSString *)layerIDString context:(PTImageIODataReadContext *)readContext;
#endif
@optional
- (BOOL)previewImageBitmapDataWasLoadedWithContext:(PTImageIODataReadContext *)readContext;

// will be deprecated
- (BOOL)needsLayerBitmapData;
- (BOOL)needsPrerenderedImageBitmapData;
- (PTImageIOConsumerNeedsPreviewImageDataResponse)needsPreviewImageBitmapData;
@end
//--------------------------------------------------------------------------------------------------------------------


