//
//  PTImageIOUtilities.h
//  OpenSaveDemo
//
//  Created by Rimas Mickevičius on 2013-01-03.
//  Copyright (c) 2013 Rimas Mickevičius. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Accelerate/Accelerate.h>

#import "PTTypes.h"

typedef NS_ENUM(NSInteger, PTSizeScaleOption) {
    PTSizeScaleAlwaysOption = 1,
    PTSizeScaleDownOnlyOption,
    PTSizeScaleUpOnlyOption,
    PTSizeScaleDefaultOption = PTSizeScaleAlwaysOption,
};

/** Logical implication: P => Q */
NS_INLINE BOOL PTImplies(BOOL P, BOOL Q) { return !P || Q; }

@interface PTImageIOUtilities : NSObject

NSString* NSStringFromPTSaveOperation(PTSaveOperation saveOperation);

+ (NSString *)generalUUIDAppendingDate:(BOOL)appendDate;
+ (NSString *)generalUUIDWithPrefix:(NSString *)prefixString;

+ (BOOL)isObjectClassSupportedAsValue:(id)objectToCheck;

+ (PTSize)fitSize:(PTSize)sizeToFit intoSize:(PTSize)sizeFitInto;
+ (PTSize)fitSize:(PTSize)sizeToFit intoSize:(PTSize)sizeFitInto options:(PTSizeScaleOption)fitOptions;
+ (PTSize)fillSize:(PTSize)sizeToFill intoSize:(PTSize)sizeFillInto;
+ (PTSize)fillSize:(PTSize)sizeToFill intoSize:(PTSize)sizeFillInto options:(PTSizeScaleOption)fillOptions;

+ (BOOL)imageHasTransparencyInData:(NSData *)imageData ofSize:(CGSize)imageSize rowBytes:(NSInteger)rowBytes alphaIndex:(uint8_t)alphaIndex bitsPerComponent:(uint8_t)bitsPerComponent;
+ (BOOL)imageHasTransparencyInVImageBuffer:(vImage_Buffer)imageVbuffer alphaIndex:(uint8_t)alphaIndex bitsPerComponent:(uint8_t)bitsPerComponent;

+ (CGRect)transparencyTrimmedRectInvImageBuffer:(vImage_Buffer *)imageBufferRef limitingRect:(CGRect)limitingRect alphaIndex:(uint8_t)alphaIndex bytesPerPixel:(uint8_t)bytesPerPixel numberOfComponents:(uint8_t)numberOfComponents;

// returns file, located at fileLocationURL, filesize in bytes. If this is beeing called from NSDocument, please provide it in document param.
+ (NSUInteger)fileSizeAtURL:(NSURL *)fileLocationURL callingFromDocument:(id <NSFilePresenter>)document;

+ (NSDictionary *)basicMetadataForFileAtURL:(NSURL *)fileURL fileType:(NSString *)fileType;

+ (BOOL)isFileAtURLBinaryPXM:(NSURL *)fileURL;
+ (BOOL)isFileAtURLPXM2:(NSURL *)fileURL;
+ (BOOL)isFileAtURLPXM16x:(NSURL *)fileURL;

+ (NSString *)typeUTIForFileAtURL:(NSURL *)fileURL;


#if PT_TARGET_OSX
#pragma - OS X versions -
+ (BOOL)isLionOrLater;
+ (BOOL)isMountainLionOrLater;
+ (BOOL)isMavericksOrLater;
+ (BOOL)isYosemiteOrLater;
+ (BOOL)isYosemite_10_10_2_OrLater;
+ (BOOL)isYosemite_10_10_3_OrLater;
#endif // PT_TARGET_OSX

@end



// temporary implementations until PTFoundation will become available
NSData* PTImageIO_DataFromDouble(double doubleToEncode);
double PTImageIO_DoubleFromData(NSData *doubleDataToDecode);
NSData* PTImageIO_DataFromCGAffineTransform(CGAffineTransform transformToEncode);
CGAffineTransform PTImageIO_CGAffineTransformFromData(NSData *cgaffineTransformDataToDecode);