//
//  PTLayerStyle.h
//  PTImageIO
//
//  Created by Rolan Reznik on 15/07/14.
//
//

#ifndef PT_LAYER_STYLE_H_INCLUDED
#define PT_LAYER_STYLE_H_INCLUDED

#import <Foundation/Foundation.h>

extern NSString *const PTLayerStyleSnapshotKey;

///NSNumber with double value in [0;100]
extern NSString *const PTLayerStyleOpacityKey;

///NSString with PTImageIOBlendModeKey
extern NSString *const PTLayerStyleBlendModeKey;
///NSDictionary
extern NSString *const PTLayerStyleFillKey;
///NSValue with BOOL value
extern NSString *const PTLayerStyleIsEnabledKey;
///NSDictionary
extern NSString *const PTLayerStyleGradientKey;
/** 
 *  Gradient points should be in normalized layer coordinates.
 *  Zero is at the top left corner.
 *  NSValue with CGPoint value.
 **/
extern NSString *const PTLayerStyleGradientLinearPoint1;
///NSValue with CGPoint value
extern NSString *const PTLayerStyleGradientLinearPoint2;
///NSValue with CGPoint value
extern NSString *const PTLayerStyleGradientRadialAndAnglePoint1;
///NSValue with CGPoint value
extern NSString *const PTLayerStyleGradientRadialAndAnglePoint2;

extern NSString *const PTLayerStyleStrokeModeKey;
extern NSString *const PTLayerStyleStrokeStyleKey;
extern NSString *const PTLayerStyleStrokePositionKey;
extern NSString *const PTLayerStyleFillModeKey;

///NSData of CGColor created with PTNSDataFromCGColor
extern NSString *const PTLayerStyleColorKey;


extern NSString *const PTLayerStyleStrokeKey;
extern NSString *const PTLayerStyleShadowKey;
extern NSString *const PTLayerStyleInnerShadowKey;

/**
 * NSNumber with double value of shadow offset
 * Shadow offset should be in pixels.
 **/
extern NSString *const PTLayerStyleShadowOffsetKey;
extern NSString *const PTLayerStyleShadowBlurKey;
/**
 * NSNumber with double value of shadow angle
 * Shadow angle should be in radians and in counterclockwise order starting at 3 o'clock.
 **/
extern NSString *const PTLayerStyleShadowAngleKey;

extern NSString *const PTLayerStyleReflectionKey;
///NSNumber with double value
extern NSString *const PTLayerStyleVersionKey;

extern NSString *const PTLayerStyleStrengthKey;
///NSValue with CGRect of layer rect in pixels
extern NSString *const PTLayerStyleLayerRectKey;

///NSValue with CGSize of document size in pixels
extern NSString *const PTLayerStyleDocumentSizeKey;

typedef NSDictionary* PTLayerStyleSnapshot;


extern const double PTLayerStyleSnapshotVersion;

typedef NS_ENUM(NSUInteger, PTLayerStyleFillMode) {
    PTLayerStyleFillModeColor,
    PTLayerStyleFillModeGradient
};

typedef NS_ENUM(NSUInteger, PTLayerStyleStrokeMode) {
    PTLayerStyleStrokeModeColor,
    PTLayerStyleStrokeModeGradient
};

typedef NS_ENUM(NSUInteger, PTLayerStyleStrokeStyle) {
    PTLayerStyleStrokeStyleFlat,
    PTLayerStyleStrokeStyleDotted,
    PTLayerStyleStrokeStyleDashed
};

typedef NS_ENUM(NSUInteger, PTLayerStyleStrokePosition) {
    PTLayerStyleStrokePositionCenter,
    PTLayerStyleStrokePositionInside,
    PTLayerStyleStrokePositionOutside
};



typedef NSDictionary* PTGradientSnapshot;


typedef NS_ENUM(NSUInteger, PTGradientType) {
    PTGradientTypeLinear,
    PTGradientTypeRadial,
    PTGradientTypeAngle
};

extern const double PTGradientSnapshotVersion;

extern NSString *const PTGradientSnapshotVersionKey;
extern NSString *const PTGradientTypeKey;
extern NSString *const PTGradientColorStopKey;
extern NSString *const PTGradientColorStopListKey;
extern NSString *const PTGradientMidpointListKey;
extern NSString *const PTGradientColorStopPositionKey;
extern NSString *const PTGradientColorStopColorKey;



#endif//PT_LAYER_STYLE_H_INCLUDED


